import json
import requests
import typer
import os
from web3.auto import w3
from eth_account.messages import encode_defunct
from py_crypto_hd_wallet import HdWalletFactory, HdWalletCoins, HdWalletSpecs, HdWalletDataTypes, HdWalletKeyTypes

app = typer.Typer()

host = "https://bxn.mml.keyko.rocks"
wallet = None
privateKey = None
accessToken = None
provider = None

class BearerAuth(requests.auth.AuthBase):
    def __init__(self, token):
        self.token = token
    def __call__(self, r):
        r.headers['Authorization'] = "Bearer " + self.token
        return r

def getNonce():
    response = requests.get(host + '/provider/' + wallet)

    if response.status_code == 200:
        try:
            return response.json()['nonce']
        except:
            return None

    return None

def signMessage(msg):
    message = encode_defunct(text=msg)
    signedMessage = w3.eth.account.sign_message(message, private_key=privateKey)

    return signedMessage.signature.hex()

def authenticate(signature):
    global provider, accessToken
    payload = {'signature': signature}
    response = requests.post(host + '/provider/auth/' + wallet, json=payload)
    provider = response.json()

    if 'accessToken' in provider:
        accessToken = provider['accessToken']

        return True
    return False

def getConfigDirectory():
    userHome = os.path.expanduser('~')
    return userHome + '/.bitscreen'

def getConfigFile():
    configDir = getConfigDirectory()
    return configDir + '/.cli_config'

def getConfigFromFile(configKey):
    cf = open(getConfigFile())
    config = json.load(cf)
    cf.close()

    if configKey not in config:
        return None

    return config[configKey]

def getAuthDataFromSeed(seed: str):
    hd_wallet_fact = HdWalletFactory(HdWalletCoins.ETHEREUM)
    hd_wallet = hd_wallet_fact.CreateFromMnemonic("_my_wallet_", seed)
    hd_wallet.Generate(addr_num=1)
    hd_wallet_key = hd_wallet.GetData(HdWalletDataTypes.ADDRESSES)[0]
    address = hd_wallet_key.GetKey(HdWalletKeyTypes.ADDRESS)
    private_key = hd_wallet_key.GetKey(HdWalletKeyTypes.RAW_PRIV)

    return address.lower(), private_key

@app.command()
def login(fromSeed: bool = False):
    global wallet, privateKey, accessToken
    configDir = getConfigDirectory()

    if not os.path.isdir(configDir):
        os.mkdir(configDir)
        typer.secho("Created bitscreen config directory.")

    configFile = getConfigFile()
    if not os.path.isfile(configFile):
        with open(configFile, 'w') as fp:
            fp.write('{}')
        typer.secho("Created bitscreen config file.")

    typer.echo(f"Checking credentials.")

    cf = open(configFile)
    config = json.load(cf)
    cf.close()

    if fromSeed:
        seed = typer.prompt("Please provide your seed phrase: ")
        try:
            wallet, privateKey = getAuthDataFromSeed(seed)
        except:
            raise typer.Exit("Invalid seed phrase.")
    else:
        if 'eth_wallet' in config:
            wallet = config['eth_wallet'].lower()
        else:
            wallet = typer.prompt("What's you Ethereum wallet address?").lower()

        if 'eth_private_key' in config:
            privateKey = config['eth_private_key']
        else:
            privateKey = typer.prompt("What's your private key?")

    nonce = getNonce()
    if nonce is None:
        raise typer.Exit("There's no account associated with this wallet.")

    try:
        signedNonce = signMessage(nonce)
    except:
        raise typer.Exit("Invalid private key.")

    isAuthenticated = authenticate(signedNonce)

    if not isAuthenticated:
        raise typer.Exit("Login failed.")

    typer.secho(f"Authenticated as " + provider['businessName'], fg=typer.colors.GREEN)

    saveCredentials = typer.confirm("Do you want to save credentials for future logins?")

    toSave = {
        'access_token': accessToken,
        'provider_id': provider['id']
    }

    if saveCredentials:
        toSave['eth_private_key'] = privateKey
        toSave['eth_wallet'] = wallet

    with open(configFile, 'w') as f:
        f.write(json.dumps(toSave))

@app.command()
def logout():
    configDir = getConfigDirectory()
    configFile = getConfigFile()

    if not os.path.isdir(configDir) or not os.path.isfile(configFile):
        typer.secho("Not logged in.")
        raise typer.Exit()

    with open(configFile, 'w') as f:
        f.write(json.dumps({}))

@app.command()
def register(wallet: str):
    response = requests.post(f'{host}/provider/{wallet}')

    if response.status_code == 200:
        typer.secho("Done. You can proceed to log in.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        print(response.json()['message'])

if __name__ == "__main__":
    app()



import json
import requests
import typer
import os
import subprocess
import re

from .auth import host, getConfigFromFile

app = typer.Typer()

@app.command()
def install():
    typer.secho("Installing BitScreen Plugin.")
    try:
        subprocess.call(["go", "install", "github.com/Murmuration-Labs/bitscreen/cmd/bitscreen@latest"], stdout=subprocess.DEVNULL)
    except OSError as e:
        print(e);
        return;

    typer.secho("Installing BitScreen Updater.")
    try:
        subprocess.call(["pip", "install", "bitscreen-updater"], stdout=subprocess.DEVNULL)
    except OSError as e:
        print(e);
        return;

    privateKey = None
    seedPhrase = None
    autoAuth = typer.confirm("Would you like to authenticate the BitScreen Updater with your CLI credentials?")
    if autoAuth:
        privateKey = getConfigFromFile("eth_private_key")
        if not privateKey:
            typer.secho("Private key not found. Maybe you didn't choose to save it.")
    if not privateKey:
        typer.secho("Proceeding with manual authentication.")
        authMethod = typer.prompt("How would you like to authenticate? [0] -> Private Key [1] -> Seed Phrase")
        if authMethod == "0":
            privateKey = typer.prompt("Please insert your private key: ")
        if authMethod == "1":
            seedPhrase = typer.prompt("Please insert your seed phrase: ")

        if not privateKey and not seedPhrase:
            raise typer.Exit("Invalid option selected")

    typer.secho("Writing environment init script.")
    typer.secho("You can find it at ~/.murmuration/updater_config.sh and execute it before using BitScreen Updater");
    typer.secho("Example: source ~/.murmuration/updater_config.sh")

    fullPath = os.path.expanduser("~/.murmuration/updater_config.sh")
    configDir = os.path.dirname(fullPath)
    if not os.path.exists(configDir):
        os.makedirs(configDir)

    with open(fullPath, "w") as f:
        f.write("export BITSCREEN_SOCKET_PORT=5555\n");
        f.write("export BITSCREEN_BACKEND_HOST=https://bxn.mml.keyko.rocks\n");
        f.write("export BITSCREEN_CIDS_FILE=~/.murmuration/bitscreen\n");
        if privateKey:
            f.write(f"export BITSCREEN_PROVIDER_KEY={privateKey}\n");
        if seedPhrase:
            f.write(f"export BITSCREEN_PROVIDER_SEED_PHRASE={seedPhrase}\n");

    typer.secho("Configuring Lotus Miner to use BitScreen Filter.")
    minerConfigPath = os.getenv('LOTUS_MINER_PATH')
    if len(minerConfigPath) == 0:
        raise typer.Exit("LOTUS_MINER_PATH environment variable not found. Could not configure BitScreen for the Lotus Miner")

    goPath = subprocess.check_output(["go", "env", "GOPATH"]).decode().strip()
    if len(goPath) == 0:
        raise typer.Exit("GOPATH environment variable not found. Could not configure BitScreen for the Lotus Miner")
    goPath += "/bin/bitscreen"

    with open(minerConfigPath + "/config.toml", "r+") as f1:
         contents = f1.read()
         contents = re.sub(r"\s\sFilter\s=\s\".*\"", f"  Filter = \"{goPath}\"", contents)
         contents = re.sub(r"\s\sRetrievalFilter\s=\s\".*\"", f"  RetrievalFilter = \"{goPath}\"", contents)

         f1.seek(0)
         f1.truncate()
         f1.write(contents)

     typer.secho("Done.")

if __name__ == "__main__":
    app()



import json
import requests
import typer
import os
from .auth import host, getConfigFromFile, BearerAuth
from dashing import *

app = typer.Typer()

state = {
    'accessToken': None,
    'providerId': None
}

@app.command()
def show():
    textColor = 2
    borderColor = 3
    ui = VSplit(
        HSplit(
            Text("Currently Filtering CIDs\n8", color=textColor, border_color=borderColor),
            Text("List subscriberss\n8", color=textColor, border_color=borderColor),
            Text("Deals declined\n8", color=textColor, border_color=borderColor),
        ),
        HSplit(
            Text("Active lists\n8", color=textColor, border_color=borderColor),
            Text("Inactive lists\n8", color=textColor, border_color=borderColor),
            Text("Imported lists\n8", color=textColor, border_color=borderColor),
            Text("Private lists\n8", color=textColor, border_color=borderColor),
            Text("Public lists\n8", color=textColor, border_color=borderColor),
        ),
        HSplit(),
        HSplit()
    )

    ui.display()

@app.callback()
def getAuthData():
    state['accessToken'] = getConfigFromFile('access_token')
    state['providerId'] = getConfigFromFile('provider_id')

    if state['accessToken'] is None or state['providerId'] is None:
        raise typer.Exit("Not logged in.")

if __name__ == "__main__":
    app()



import json
import requests
import typer
import os
from tabulate import tabulate
from .auth import host, getConfigFromFile, BearerAuth
from typing import Optional

VISIBILITY_TYPES = {
    1: 'Private',
    2: 'Public',
    3: 'Shareable'
}

app = typer.Typer()

state = {
    'accessToken': None,
    'providerId': None
}

def getFilters(params = {}):
    params['providerId'] = state['providerId'];

    response = requests.get(host + '/filter', params=params, auth=BearerAuth(state['accessToken']))
    filters = response.json()

    return filters

def getFilterDetails(filterId, params = {}):
    params['providerId'] = state['providerId'];

    response = requests.get(host + '/filter/' + filterId, params=params, auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        filters = response.json()
        return filters

    raise typer.Exit("Filter not found.")

def getReadableVisibility(visibilityId):
    return VISIBILITY_TYPES[visibilityId]

def parseVisibilityCallback(value: str):
    if value is None:
        return None

    for key in VISIBILITY_TYPES:
        if VISIBILITY_TYPES[key].lower() == value.lower():
            return key
    raise typer.BadParameter("Invalid visibility type. Allowed types are: Private, Public, Shareable")

def parseOverrideCallback(value: int):
    if value is None:
        return None

    if value >= 0 and value <= 1:
        return value
    raise typer.BadParameter("Invalid override value. Allowed types are: 0, 1")

def printFilterLists(filterList):
    headers = ["ID", "Name", "Visibility", "Status", "Subscribers", "CIDs", "Override", "Provider", "Description"]
    rows = [];
    for filter in filterList:
        rows.append([
            filter['shareId'],
            filter['name'],
            getReadableVisibility(filter['visibility']),
            'Enabled' if filter['enabled'] else 'Disabled',
            len(filter['provider_Filters']),
            filter['cidsCount'],
            'Yes' if filter['override'] else 'No',
            filter['provider']['businessName'],
            filter['description']
        ])
    print(tabulate(rows, headers, tablefmt="fancy_grid"))

def printCidLists(cidList):
    headers = ["CID", "Reference URL", "Created", "Updated"]
    rows = []

    for cid in cidList:
        rows.append([
            cid['cid'],
            cid['refUrl'],
            cid['created'],
            cid['updated']
        ])

    print(tabulate(rows, headers, tablefmt="fancy_grid"))

def printFilterDetails(filter):
    if filter['enabled']:
        typer.secho("Enabled", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Disabled", bg=typer.colors.RED)
    typer.secho(f"Filter name:  {filter['name']}")
    typer.secho(f"Description: {filter['description']}")
    typer.secho(f"ID: {filter['shareId']}")
    typer.secho(f"Visibility: {getReadableVisibility(filter['visibility'])}")
    typer.secho(f"Subscribers: {len(filter['provider_Filters'])}")
    typer.secho(f"Override: {('Yes' if filter['override'] else 'No')}")
    typer.secho(f"Owner: {filter['provider']['businessName']}")
    typer.secho(f"CID count: {len(filter['cids'])}")

    printCidLists(filter['cids'])

def setFilterStatus(filter: str, status: bool):
    filterDetails = getFilterDetails(filter)

    allowed = False
    for providerFilter in filterDetails['provider_Filters']:
        if providerFilter['provider']['id'] == state['providerId']:
            allowed = True
            if providerFilter['active'] == status:
                raise typer.Exit("Status already set.")

    if allowed:
        params = {'active': status}
        response = requests.put(f"{host}/provider-filter/{state['providerId']}/{filterDetails['id']}", json=params, auth=BearerAuth(state['accessToken']))
        if response.status_code == 200:
            typer.secho("Done.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
        else:
            typer.secho("Error: ", bg=typer.colors.RED)
            typer.secho(response.json())

@app.command()
def list(search: str = ""):
    params = {};
    if len(search) > 0:
        params['q'] = search
    filters = getFilters(params)

    print("Found " + str(filters['count']) + " filters:")

    printFilterLists(filters['filters'])

@app.command()
def details(filter: str):
    filterDetails = getFilterDetails(filter)
    printFilterDetails(filterDetails)

@app.command()
def enable(filter: str):
    setFilterStatus(filter, True)

@app.command()
def disable(filter: str):
    setFilterStatus(filter, False)

@app.command()
def edit(
    filter: str,
    name: str = None,
    description: str = None,
    override: int = typer.Option(None, callback=parseOverrideCallback),
    visibility: str = typer.Option(None, callback=parseVisibilityCallback)
):
    filter = getFilterDetails(filter)
    if name is not None:
        filter['name'] = name

    if description is not None:
        filter['description'] = description

    if override is not None:
        filter['override'] = (override == 1)

    if visibility is not None:
        filter['visibility'] = visibility

    response = requests.put(f"{host}/filter/{filter['id']}", json=filter, auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Done.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        typer.secho(response.json())

@app.command()
def add(
    name: str = typer.Option(..., prompt=True),
    description: str = typer.Option(..., prompt=True),
    visibility: str = typer.Option(..., prompt="What visibility should the filter have? [Private/Public/Shareable]", callback=parseVisibilityCallback),
    override: int = typer.Option(..., prompt="Is this an override filter? [0/1]", callback=parseOverrideCallback),
):
    filter = {
        'cids': [],
        'enabled': True,
        'name': name,
        'description': description,
        'visibility': visibility,
        'override': (override == 1),
        'providerId': state['providerId'],
    }

    response = requests.post(f"{host}/filter", json=filter, auth=BearerAuth(state['accessToken']))

    filter = response.json()

    providerFilter = {
        'active': True,
        'filterId': filter['id'],
        'providerId': state['providerId']
    }
    response = requests.post(f"{host}/provider-filter", json=providerFilter, auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Done.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
        filter['provider_Filters'] = []
        filter['cids'] = []
        printFilterDetails(filter)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        typer.secho(response.json())


@app.command(name="add-cid")
def add_cid(filter: str, cid: str, refUrl: str = ""):
    filter = getFilterDetails(filter)
    cid = {
        'cid': cid,
        'filterId': filter['id'],
        'refUrl': refUrl
    }

    response = requests.post(f"{host}/cid", json=cid, auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Done.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        typer.secho(response.json())

@app.command(name="remove-cid")
def remove_cid(filter: str, cid: str):
    filter = getFilterDetails(filter)
    id = None
    for cidObj in filter['cids']:
        if cidObj['cid'] == cid:
            id = cidObj['id']
            break

    if id is None:
        raise typer.Exit("CID not found on provided filter.")

    response = requests.delete(f"{host}/cid/{id}", auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Done.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        typer.secho(response.json())

@app.command()
def delete(filter: str, confirm: bool = False):
    filter = getFilterDetails(filter)

    providerFilter = None
    for providerFilter in filter['provider_Filters']:
        if providerFilter['provider']['id'] == state['providerId']:
            providerFilter = providerFilter['id']
            break

    if providerFilter is None:
        raise typer.Exit("You do not own or import this filter.");

    if not confirm:
        confirm = typer.confirm(f"Are you sure you want to delete filter {filter['name']}?")

    if not confirm:
        raise typer.Exit("Aborting.");

    response = requests.delete(f"{host}/provider-filter/{state['providerId']}/{filter['id']}", auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Deleted.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        typer.secho(response.json())

@app.callback()
def getAuthData():
    state['accessToken'] = getConfigFromFile('access_token')
    state['providerId'] = getConfigFromFile('provider_id')

    if state['accessToken'] is None or state['providerId'] is None:
        raise typer.Exit("Not logged in.")

if __name__ == "__main__":
    app()



import json
import requests
import typer
import os
from .auth import host, getConfigFromFile, BearerAuth
from typing import Optional

state = {
    'accessToken': None,
    'providerId': None
}

app = typer.Typer()

def getCids(download = False):
    query = {
        download: download
    }

    response = requests.get(host + '/cid/blocked', params=query, auth=BearerAuth(state['accessToken']))

    if download:
        return response.content

    cids = response.json()

    return cids

@app.command()
def list(
    outputFile: str = typer.Option(None, "--outputfile", "-o")
):
    if outputFile:
        cids = getCids(True)
        open(outputFile, 'wb').write(cids)
        raise typer.Exit("CID list written at: " + outputFile)

    cids = getCids(False)

    print("Found " + str(len(cids)) + " blocked CIDs:")

    for cid in cids:
        typer.secho(cid)

@app.callback()
def getAuthData():
    state['accessToken'] = getConfigFromFile('access_token')
    state['providerId'] = getConfigFromFile('provider_id')

    if state['accessToken'] is None or state['providerId'] is None:
        raise typer.Exit("Not logged in.")

if __name__ == "__main__":
    app()



import json
import requests
import typer
import os
from tabulate import tabulate
from .auth import host, getConfigFromFile, BearerAuth
from .filter import getReadableVisibility

app = typer.Typer()

state = {
    'accessToken': None,
    'providerId': None
}

def getPublicFilters(params = {}):
    params['providerId'] = state['providerId'];

    response = requests.get(host + '/filter/public', params=params, auth=BearerAuth(state['accessToken']))
    filters = response.json()

    return filters['data']

def getPublicFilterDetails(filterId, params = {}):
    params['providerId'] = state['providerId'];

    response = requests.get(host + '/filter/public/details/' + filterId, params=params, auth=BearerAuth(state['accessToken']))

    if response.status_code == 200:
        data = response.json()
        data['filter']['isImported'] = data['isImported']
        data['filter']['provider'] = data['provider']
        return data['filter']

    raise typer.Exit("Filter not found.")

def getStatusFromFilter(filter):
    status = typer.style("Not imported", fg=typer.colors.WHITE, bg=typer.colors.RED)
    if filter['isImported']:
        status = typer.style("Imported", fg=typer.colors.GREEN, bold=True)
    elif filter['provider']['id'] == state['providerId']:
        status = typer.style("Owned", fg=typer.colors.GREEN)

    return status

def printPublicFilterLists(filterList: list):
    headers = ["ID", "Name", "Visibility", "Status", "Subscribers", "CIDs", "Override", "Provider", "Description"]
    rows = [];
    for filter in filterList:
        status = getStatusFromFilter(filter)

        rows.append([
            filter['shareId'],
            filter['name'],
            getReadableVisibility(filter['visibility']),
            status,
            filter['subs'],
            filter['cids'],
            'Yes' if filter['override'] else 'No',
            filter['provider']['businessName'],
            filter['description']
        ])
    print(tabulate(rows, headers, tablefmt="fancy_grid"))

def printPublicFilterDetails(filter):
    typer.echo(getStatusFromFilter(filter))
    typer.secho(f"Filter name:  {filter['name']}")
    typer.secho(f"Description: {filter['description']}")
    typer.secho(f"ID: {filter['shareId']}")
    typer.secho(f"Subscribers: {len(filter['provider_Filters'])}")
    typer.secho(f"Override: {('Yes' if filter['override'] else 'No')}")
    typer.secho(f"CID count: {len(filter['cids'])}")
    typer.secho(f"Business name: {filter['provider']['businessName']}")
    typer.secho(f"Contact person: {filter['provider']['contactPerson']}")
    typer.secho(f"Website: {filter['provider']['website']}")
    typer.secho(f"Email: {filter['provider']['businessName']}")
    typer.secho(f"Address: {filter['provider']['businessName']}")
    typer.secho(f"City & Country: {filter['provider']['businessName']}")

@app.command()
def list(search: str = ""):
    params = {};
    if len(search) > 0:
        params['q'] = search
    filters = getPublicFilters(params)

    typer.secho(f"Found {len(filters)} filters.")
    printPublicFilterLists(filters)

@app.command()
def details(filter: str):
    filter = getPublicFilterDetails(filter)

    printPublicFilterDetails(filter)

@app.command(name="import")
def importFilter(filter: str):
    filter = getPublicFilterDetails(filter)
    if filter['isImported']:
        raise typer.Exit("Filter already imported.")

    if filter['provider']['id'] == state['providerId']:
        raise typer.Exit("Cannot import your own filter.")

    if filter['visibility'] == 1:
        raise typer.Exit("Filter is not public or shareable.")

    providerFilter = {
        'active': True,
        'filterId': filter['id'],
        'providerId': state['providerId']
    }
    response = requests.post(f"{host}/provider-filter", json=providerFilter, auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Imported.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        typer.secho(response.json())

@app.command(name="discard")
def discardFilter(filter: str):
    filter = getPublicFilterDetails(filter)

    if not filter['isImported']:
        raise typer.Exit("This filter is not imported.")

    response = requests.delete(f"{host}/provider-filter/{state['providerId']}/{filter['id']}", auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Discarded.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        typer.secho(response.json())

@app.callback()
def getAuthData():
    state['accessToken'] = getConfigFromFile('access_token')
    state['providerId'] = getConfigFromFile('provider_id')

    if state['accessToken'] is None or state['providerId'] is None:
        raise typer.Exit("Not logged in.")

if __name__ == "__main__":
    app()



#!/usr/bin/env python

import json
import requests
import typer
import os

from .commands import auth, filter, directory, dashboard, settings, cid, setup

app = typer.Typer()

app.add_typer(auth.app, name="auth")
app.add_typer(filter.app, name="filter")
app.add_typer(directory.app, name="directory")
app.add_typer(dashboard.app, name="dashboard")
app.add_typer(settings.app, name="settings")
app.add_typer(cid.app, name="cid")
app.add_typer(setup.app, name="setup")



import json
import requests
import typer
import os
from enum import Enum
from tabulate import tabulate
from .auth import host, getConfigFromFile, BearerAuth

app = typer.Typer()

state = {
    'accessToken': None,
    'providerId': None,
    'wallet': None
}

class Action(str, Enum):
    Filtering = "filtering",
    Sharing = "sharing",
    Importing = "importing"

class ProviderDataKey(str, Enum):
    Country = "country",
    BusinessName = "name",
    Website = "website",
    Email = "email",
    ContactPerson = "contact-person",
    Address = "address"

def mapProviderDataKey(key: ProviderDataKey):
    return {
        ProviderDataKey.Country: "country",
        ProviderDataKey.BusinessName: "businessName",
        ProviderDataKey.Website: "website",
        ProviderDataKey.Email: "email",
        ProviderDataKey.ContactPerson: "contactPerson",
        ProviderDataKey.Address: "address",
    }[key]

def setValue(action: Action, value: bool):
    response = requests.get(f"{host}/config/{state['providerId']}", auth=BearerAuth(state['accessToken']))
    if response.status_code != 200:
        typer.secho("Error: ", bg=typer.colors.RED)
        typer.secho(response.json())
        raise typer.Exit()

    config = response.json()

    key = None
    if action is Action.Filtering:
        key = 'bitscreen'
    if action is Action.Sharing:
        key = 'share'
    if action is Action.Importing:
        key = 'import'

    if key is None:
        raise typer.Exit("Invalid action")

    if config[key] == value:
        raise typer.Exit("Value already set.")

    config['providerId'] = state['providerId']
    config[key] = value;
    response = requests.put(f"{host}/config", json=config, auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Done.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        print(response)

@app.command()
def enable(
    action: Action = typer.Argument(..., case_sensitive=False)
):
    setValue(action, True)

@app.command()
def disable(
    action: Action = typer.Argument(..., case_sensitive=False)
):
    setValue(action, False)

@app.command(name="set")
def setProviderData(
    key: ProviderDataKey = typer.Argument(..., case_sensitive=False),
    value: str = typer.Argument(...)
):
    response = requests.get(f"{host}/provider/{state['wallet']}")
    provider = response.json()

    provider[mapProviderDataKey(key)] = value
    provider['walletAddress'] = state['wallet']
    response = requests.put(f"{host}/provider", json=provider, auth=BearerAuth(state['accessToken']))
    if response.status_code == 200:
        typer.secho("Done.", bg=typer.colors.GREEN, fg=typer.colors.BLACK)
    else:
        typer.secho("Error: ", bg=typer.colors.RED)
        print(response.json())

@app.command()
def get():
    response = requests.get(f"{host}/provider/{state['wallet']}")
    provider = response.json()

    typer.secho(f"Business Name: {provider['businessName']}")
    typer.secho(f"Country: {provider['country']}")
    typer.secho(f"Website: {provider['website']}")
    typer.secho(f"Email: {provider['email']}")
    typer.secho(f"Contact Person: {provider['contactPerson']}")
    typer.secho(f"Address: {provider['address']}")

@app.callback()
def getAuthData():
    state['accessToken'] = getConfigFromFile('access_token')
    state['providerId'] = getConfigFromFile('provider_id')
    state['wallet'] = getConfigFromFile('eth_wallet')

    if state['accessToken'] is None or state['providerId'] is None:
        raise typer.Exit("Not logged in.")

if __name__ == "__main__":
    app()
